@@ -57,7 +57,7 @@ clear_all()
 int
 init_mem()
 {
-	if ((scratch = (token_type *) malloc((n_tokens + 10) * sizeof(token_type))) == NULL
+	if ((scratch = (token_type *) malloc(((n_tokens * 3) / 2) * sizeof(token_type))) == NULL
 	    || (tlhs = (token_type *) malloc(n_tokens * sizeof(token_type))) == NULL
 	    || (trhs = (token_type *) malloc(n_tokens * sizeof(token_type))) == NULL) {
 		return false;
@@ -78,7 +78,7 @@ init_gvars()
 	partial_flag = true;
 	symb_flag = false;
 	sign_flag = false;
-	in_calc_cmd = false;
+	approximate_roots = false;
 
 	zero_token.level = 1;
 	zero_token.kind = CONSTANT;
@@ -282,7 +282,7 @@ token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to equation side length */
 token_type	*expression;	/* expression pointer */
 int		len;		/* expression length */
-long		v;
+long		v;		/* variable to substitute with expression */
 {
 	int	j, k;
 	int	level;
@@ -487,7 +487,9 @@ int	i;	/* equation space number */
 }
 
 /*
- * Return true and display a message if current equation is undefined.
+ * Verify that a current equation or expression is loaded.
+ *
+ * Return true and display a message if current equation space is empty.
  */
 int
 current_not_defined()
@@ -528,7 +530,7 @@ int	n;		/* maximum size of "string" in bytes */
 	}
 	input_column = strlen(prompt_str);
 #if	READLINE
-	if (!html_flag) {
+	if (readline_enabled) {
 		cp = readline(prompt_str);
 		if (cp == NULL)
 			exit_program(0);
@@ -580,20 +582,26 @@ get_yes_no()
 
 /*
  * Store or display the result of a command.
+ *
+ * Return true if successful.
  */
+int
 return_result(en)
 int	en;	/* equation space number */
 {
 #if	LIBRARY
 	if (gfp == stdout) {
+		if (display2d) {
+			group_sub(en);
+		}
 		if (factor_int_flag) {
 			factor_int_sub(en);
 		}
 		result_str = list_equation(en, false);
-		return;
+		return(result_str != NULL);
 	}
 #endif
-	list_sub(en);
+	return list_sub(en);
 }
 
 /*
@@ -737,7 +745,7 @@ char	*cp;	/* character pointer */
 
 /*
  * Enhanced decimal strtol().
- * Allows and skips trailing spaces.
+ * Skips trailing spaces.
  */
 long
 decstrtol(cp, cpp)
@@ -762,13 +770,12 @@ char	*
 skip_param(cp)
 char	*cp;	/* character pointer */
 {
-	while (*cp && !isspace(*cp) && *cp != '=') {
+	while (*cp && (!isascii(*cp) || (!isspace(*cp) && *cp != '='))) {
 		cp++;
 	}
 	cp = skip_space(cp);
 	if (*cp == '=') {
-		cp++;
-		cp = skip_space(cp);
+		cp = skip_space(cp + 1);
 	}
 	return(cp);
 }
@@ -777,6 +784,8 @@ char	*cp;	/* character pointer */
  * Compare strings up to the first space.
  * Must be an exact match.
  * Compare is alphabetic case insensitive.
+ *
+ * Returns zero on match, non-zero if different.
  */
 int
 strcmp_tospace(cp1, cp2)
@@ -795,7 +804,7 @@ char	*cp1, *cp2;
  * Return the number of "level" additive type operators.
  */
 int
-level_plus(p1, n1, level)
+level_plus_count(p1, n1, level)
 token_type	*p1;	/* expression pointer */
 int		n1;	/* expression length */
 int		level;	/* parentheses level number */
@@ -820,14 +829,11 @@ int		level;	/* parentheses level number */
  * Return the number of level 1 additive type operators.
  */
 int
-level1_plus(p1, n1)
+level1_plus_count(p1, n1)
 token_type	*p1;	/* expression pointer */
 int		n1;	/* expression length */
 {
-	int	level;
-
-	level = min_level(p1, n1);
-	return level_plus(p1, n1, level);
+	return level_plus_count(p1, n1, min_level(p1, n1));
 }
 
 /*
@@ -850,6 +856,41 @@ int		n1;	/* expression length */
 	return count;
 }
 
+/*
+ * Set *vp if single variable expression.
+ *
+ * Return true if expression contains no variables.
+ */
+int
+no_vars(source, n, vp)
+token_type	*source;	/* expression pointer */
+int		n;		/* expression length */
+long		*vp;		/* variable pointer */
+{
+	int	j;
+	int	found = false;
+
+	if (*vp) {
+		return(var_count(source, n) == 0);
+	}
+	for (j = 0; j < n; j += 2) {
+		if (source[j].kind == VARIABLE) {
+			if ((source[j].token.variable & VAR_MASK) <= SIGN)
+				continue;
+			if (*vp) {
+				if (*vp != source[j].token.variable) {
+					*vp = 0;
+					break;
+				}
+			} else {
+				found = true;
+				*vp = source[j].token.variable;
+			}
+		}
+	}
+	return(!found);
+}
+
 /*
  * Return true if expression contains infinity or nan.
  */
