@@ -1,5 +1,5 @@
 /*
- * C code for the Mathomatic commands.
+ * Mathomatic commands that don't belong anywhere else.
  *
  * Copyright (C) 1987-2006 George Gesslein II.
  */
@@ -9,17 +9,18 @@
 #define	OPT_MIN_SIZE	7	/* Minimum size of repeated expressions to find in optimize command. */
 
 static int	sum_product();
-static int	find_more();
-static int	opt_es();
 static int	complex_func();
 static int	elim_sub();
+#if	!LIBRARY
 static int	edit_sub();
+static int	find_more();
+static int	opt_es();
 
 /* Global variables for the optimize command. */
 static int	opt_en[N_EQUATIONS];
 static int	last_temp_var = 0;
+#endif
 
-#if	!LIBRARY
 /*
  * The version command.
  */
@@ -31,6 +32,9 @@ char	*cp;	/* the command line argument */
 		/* if anything on the command line, fail */
 		return false;
 	}
+#if	LIBRARY
+	result_str = strdup(VERSION);
+#endif
 	return version_report();
 }
 
@@ -56,6 +60,9 @@ version_report()
 #if	SILENT
 	fprintf(gfp, "SILENT ");
 #endif
+#if	LIBRARY
+	fprintf(gfp, "LIBRARY ");
+#endif
 #if	SECURE
 	fprintf(gfp, "SECURE ");
 #endif
@@ -68,7 +75,6 @@ version_report()
 	fprintf(gfp, "\n");
 	return true;
 }
-#endif
 
 /*
  * The solve command.
@@ -80,11 +86,13 @@ char	*cp;
 	int	i;
 	char	buf[MAX_CMD_LEN];
 
-	if (current_not_defined()) {
+	i = cur_equation;
+	if (i < 0 || i >= n_equations || n_lhs[i] <= 0 || n_rhs[i] <= 0) {
+		error(_("No current equation."));
 		return false;
 	}
 	if (*cp == '\0') {
-		my_strlcpy(prompt_str, _("Enter variable: "), sizeof(prompt_str));
+		my_strlcpy(prompt_str, _("Enter variable or 0: "), sizeof(prompt_str));
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
@@ -97,8 +105,7 @@ char	*cp;
 		if (n_lhs[i] && n_rhs[i]) {
 			error(_("Invalid argument."));
 		} else if (!extra_garbage(cp) && solve(i, cur_equation)) {
-			return_result(cur_equation);
-			return true;
+			return return_result(cur_equation);
 		}
 	}
 	n_lhs[i] = 0;
@@ -134,7 +141,7 @@ sum_product(cp, product_flag)
 char	*cp;		/* the command line */
 int	product_flag;	/* true for product, otherwise sum */
 {
-	int		i, j;
+	int		i;
 	long		v = 0;
 	double		start, end, step = 1.0;
 	int		result_en;
@@ -156,37 +163,31 @@ int	product_flag;	/* true for product, otherwise sum */
 		source = lhs[cur_equation];
 		dest = lhs[result_en];
 	}
-	if (*cp == '\0') {
-		for (j = 0; j < ns; j += 2) {
-			if (source[j].kind == VARIABLE) {
-				if ((source[j].token.variable & VAR_MASK) <= SIGN)
-					continue;
-				if (v) {
-					if (v != source[j].token.variable) {
-						v = 0;
-						break;
-					}
-				} else {
-					v = source[j].token.variable;
-				}
-			}
-		}
-		if (v == 0) {
-			if (!prompt_var(&v))
-				return false;
-		}
-	} else {
+	if (*cp) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
 		}
 	}
+	if (no_vars(source, ns, &v)) {
+		error(_("Current expression contains no variables."));
+		return false;
+	}
+	if (v == 0) {
+		if (!prompt_var(&v)) {
+			return false;
+		}
+	}
 	if (!found_var(source, ns, v)) {
 		error(_("Variable not found."));
 		return false;
 	}
 	if (*cp) {
-		cp1 = cp;
+		if (*cp == '=') {
+			cp1 = skip_space(cp + 1);
+		} else {
+			cp1 = cp;
+		}
 	} else {
 		list_var(v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), "%s = ", var_str);
@@ -199,6 +200,9 @@ int	product_flag;	/* true for product, otherwise sum */
 		return false;
 	}
 	cp = skip_space(cp);
+	if (strcmp_tospace(cp, "to") == 0) {
+		cp = skip_param(cp);
+	}
 	if (*cp) {
 		cp1 = cp;
 	} else {
@@ -224,7 +228,7 @@ int	product_flag;	/* true for product, otherwise sum */
 		return false;
 	count_down = (end < start);
 	if (fmod(fabs(start - end) / step, 1.0)) {
-		printf(_("Warning: end value not reached.\n"));
+		error(_("Warning: end value not reached."));
 	}
 	if (product_flag) {
 		dest[0] = one_token;
@@ -269,8 +273,7 @@ int	product_flag;	/* true for product, otherwise sum */
 	} else {
 		n_lhs[result_en] = n;
 	}
-	return_result(result_en);
-	return true;
+	return return_result(result_en);
 }
 
 #if	!LIBRARY
@@ -342,6 +345,7 @@ int		*np;
 	int	diff_sign;
 	int	found_se, found_se1;
 	long	v;
+	char	temp_buf[50];
 
 	found_se = true;
 	for (level = 1; found_se; level++) {
@@ -370,11 +374,14 @@ int		*np;
 					k1 = i1 - 1;
 					if ((jj1 - k1) >= OPT_MIN_SIZE
 					    && se_compare(&equation[k], j - k, &equation[k1], jj1 - k1, &diff_sign)) {
-						if (last_temp_var > MAX_SUBSCRIPT) {
+						snprintf(temp_buf, sizeof(temp_buf), "temp%d", last_temp_var);
+						if (parse_var(&v, temp_buf) == NULL) {
+							return false;	/* can't create "temp" variable */
+						}
+						last_temp_var++;
+						if (last_temp_var < 0) {
 							last_temp_var = 0;
 						}
-						v = V_TEMP + (((long) last_temp_var) << VAR_SHIFT);
-						last_temp_var++;
 						i2 = next_espace();
 						lhs[i2][0].level = 1;
 						lhs[i2][0].kind = VARIABLE;
@@ -540,6 +547,11 @@ char	*cp;
  */
 output_options()
 {
+	if (!autosolve) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "autosolve\n");
+
 #if	!SILENT
 	fprintf(gfp, "debug_level = %d\n", debug_level);
 #endif
@@ -559,7 +571,7 @@ output_options()
 	}
 	fprintf(gfp, "bold_colors\n");
 
-	if (!groupall) {
+	if (!display2d) {
 		fprintf(gfp, "no ");
 	}
 	fprintf(gfp, "display2d\n");
@@ -609,7 +621,9 @@ int
 set_options(cp)
 char	*cp;
 {
+	int	i;
 	int	negate;
+	char	*cp1;
 
 next_option:
 	cp = skip_space(cp);
@@ -636,11 +650,13 @@ next_option:
 		if (negate) {
 			debug_level = 0;
 		} else {
-			if (*cp == '\0') {
-				error(_("Please specify a debug level number."));
+			i = decstrtol(cp, &cp1);
+			if (cp == cp1) {
+				error(_("Please specify the debug level number."));
 				return false;
 			}
-			debug_level = decstrtol(cp, &cp);
+			cp = cp1;
+			debug_level = i;
 		}
 		goto next_option;
 	}
@@ -650,14 +666,20 @@ next_option:
 		if (negate) {
 			screen_columns = 0;
 		} else {
-			if (*cp == '\0') {
+			if ((i = decstrtol(cp, &cp1)) < 0 || cp == cp1) {
 				error(_("Please specify how wide the screen is in columns."));
 				return false;
 			}
-			screen_columns = decstrtol(cp, &cp);
+			cp = cp1;
+			screen_columns = i;
 		}
 		goto next_option;
 	}
+	if (strncasecmp(cp, "autosolve", 9) == 0) {
+		cp = skip_param(cp);
+		autosolve = !negate;
+		goto next_option;
+	}
 	if (strncasecmp(cp, "case", 4) == 0) {
 		cp = skip_param(cp);
 		case_sensitive_flag = !negate;
@@ -665,7 +687,7 @@ next_option:
 	}
 	if (strncasecmp(cp, "display2d", 9) == 0) {
 		cp = skip_param(cp);
-		groupall = !negate;
+		display2d = !negate;
 		goto next_option;
 	}
 	if (strncasecmp(cp, "preserve", 8) == 0) {
@@ -850,8 +872,7 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 	}
 	*np = n1;
 	cur_equation = j;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
 /*
@@ -892,6 +913,7 @@ char	*cp;
 	for (count = 0.0;; count++) {
 		fprintf(gfp, _("Running total = "));
 		list_proc(trhs, n_trhs, false);
+		fprintf(gfp, "\n");
 		if (count > 0.0) {
 			/* calculate and display the average */
 			blt(tlhs, trhs, n_trhs * sizeof(token_type));
@@ -911,10 +933,10 @@ char	*cp;
 			tlhs[n_tlhs].token.constant = count;
 			n_tlhs++;
 			calc_simp(tlhs, &n_tlhs);
-			fprintf(gfp, _(", average = "));
+			fprintf(gfp, _("Average = "));
 			list_proc(tlhs, n_tlhs, false);
+			fprintf(gfp, "\n");
 		}
-		fprintf(gfp, "\n");
 		my_strlcpy(prompt_str, _("Enter value: "), sizeof(prompt_str));
 		if (!get_expr(tlhs, &n_tlhs)) {
 			break;
@@ -1361,12 +1383,8 @@ char	*cp;
 	}
 	if (parse_complex(rhs[i], nr, &c1) && parse_complex(lhs[i], nl, &c2)) {
 		c3 = complex_div(c1, c2);
-		fprintf(gfp, _("Result of complex division: "));
-		if (c3.im == 0.0) {
-			fprintf(gfp, "%.12g\n\n", c3.re);
-		} else {
-			fprintf(gfp, "%.12g%+.12g*i#\n\n", c3.re, c3.im);
-		}
+		fprintf(gfp, _("Result of complex number division: "));
+		fprintf(gfp, "%.*g%+.*g*i#\n\n", precision, c3.re, precision, c3.im);
 		return true;
 	}
 	v_tmp = v;
@@ -1413,56 +1431,60 @@ char	*cp;
 	char	used[N_EQUATIONS];
 	int	vc = 0;
 
+	if (*cp == '\0') {
+		error(_("Please specify the variables to eliminate or \"all\" for all variables."));
+		return false;
+	}
 	for (i = 0; i < ARR_CNT(used); i++)
 		used[i] = false;
 	if (current_not_defined()) {
 		return false;
 	}
-	if (is_all(cp)) {
-		cp = skip_param(cp);
-		if (extra_garbage(cp))
-			return false;
-		last_v = 0;
-		for (;;) {
-			v1 = -1;
-			for (i = 0; i < n_lhs[cur_equation]; i += 2) {
-				if (lhs[cur_equation][i].kind == VARIABLE
-				    && lhs[cur_equation][i].token.variable > last_v) {
-					if (v1 == -1 || lhs[cur_equation][i].token.variable < v1) {
-						v1 = lhs[cur_equation][i].token.variable;
-					}
-				}
-			}
-			for (i = 0; i < n_rhs[cur_equation]; i += 2) {
-				if (rhs[cur_equation][i].kind == VARIABLE
-				    && rhs[cur_equation][i].token.variable > last_v) {
-					if (v1 == -1 || rhs[cur_equation][i].token.variable < v1) {
-						v1 = rhs[cur_equation][i].token.variable;
-					}
-				}
-			}
-			if (v1 == -1)
-				break;
-			last_v = v1;
-			if ((v1 & VAR_MASK) > SIGN) {
-				if (vc >= ARR_CNT(va)) {
-					break;
-				}
-				va[vc++] = v1;
-			}
-		}
-	}
 next_var:
-	if (*cp) {
+	if (vc) {
+		v = va[--vc];
+	} else if (*cp) {
+		if (is_all(cp)) {
+			cp = skip_param(cp);
+			vc = 0;
+			last_v = 0;
+			for (;;) {
+				v1 = -1;
+				for (i = 0; i < n_lhs[cur_equation]; i += 2) {
+					if (lhs[cur_equation][i].kind == VARIABLE
+					    && lhs[cur_equation][i].token.variable > last_v) {
+						if (v1 == -1 || lhs[cur_equation][i].token.variable < v1) {
+							v1 = lhs[cur_equation][i].token.variable;
+						}
+					}
+				}
+				for (i = 0; i < n_rhs[cur_equation]; i += 2) {
+					if (rhs[cur_equation][i].kind == VARIABLE
+					    && rhs[cur_equation][i].token.variable > last_v) {
+						if (v1 == -1 || rhs[cur_equation][i].token.variable < v1) {
+							v1 = rhs[cur_equation][i].token.variable;
+						}
+					}
+				}
+				if (v1 == -1)
+					break;
+				last_v = v1;
+				if ((v1 & VAR_MASK) > SIGN) {
+					if (vc >= ARR_CNT(va)) {
+						break;
+					}
+					va[vc++] = v1;
+				}
+			}
+			goto next_var;
+		}
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
 		}
-	} else if (vc) {
-		v = va[--vc];
 	} else {
 		if (did_something) {
-			return_result(cur_equation);
+			did_something = return_result(cur_equation);
 		} else {
 			error(_("No substitutions made."));
 		}
@@ -1508,7 +1530,6 @@ next_var:
 		}
 	}
 	did_something = true;
-	simp_sub(cur_equation);
 	used[i] = true;
 	goto next_var;
 }
@@ -1523,7 +1544,7 @@ long	v;
 #if	!SILENT
 	if (debug_level >= 0) {
 		list_var(v, 0);
-		printf(_("Solving equation #%d for (%s)...\n"), i + 1, var_str);
+		printf(_("Solving equation #%d for (%s) and substituting into the current equation...\n"), i + 1, var_str);
 	}
 #endif
 	want.level = 1;
@@ -1535,11 +1556,12 @@ long	v;
 	}
 	subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], rhs[i], n_rhs[i], v);
 	subst_var_with_exp(lhs[cur_equation], &n_lhs[cur_equation], rhs[i], n_rhs[i], v);
+	simp_sub(cur_equation);
 	return true;
 }
 
 /*
- * The flist command.
+ * The flist or display command.
  *
  * Displays equations in fraction format.
  */
@@ -1566,7 +1588,7 @@ char	*cp;
 #if	LIBRARY
 			if (gfp == stdout) {
 				result_str = list_equation(i, false);
-				return true;
+				return(result_str != NULL);
 			}
 #endif
 			flist_sub(i);
@@ -1588,7 +1610,8 @@ char	*cp;
 	if (strcmp_tospace(cp, "export") == 0) {
 		export_flag = 2;
 		cp = skip_param(cp);
-	} else if (strcmp_tospace(cp, "maxima") == 0) {
+	}
+	if (strcmp_tospace(cp, "maxima") == 0) {
 		export_flag = 1;
 		cp = skip_param(cp);
 	}
@@ -1602,7 +1625,7 @@ char	*cp;
 #if	LIBRARY
 		if (gfp == stdout) {
 			result_str = list_equation(i, export_flag);
-			return true;
+			return(result_str != NULL);
 		}
 #endif
 		list1_sub(i, export_flag);
@@ -1620,9 +1643,9 @@ char	*cp;
 {
 	int	i, j;
 	int	language = 0;
-	int	int_flag = false;
+	int	int_flag = false, displayed = false;
 
-	if (strcmp_tospace(cp, "c") == 0) {
+	if (strcmp_tospace(cp, "c") == 0 || strcmp_tospace(cp, "c++") == 0) {
 		cp = skip_param(cp);
 	} else if (strcmp_tospace(cp, "java") == 0) {
 		cp = skip_param(cp);
@@ -1640,30 +1663,27 @@ char	*cp;
 	for (; i <= j; i++) {
 		if (n_lhs[i] > 0) {
 			if (n_rhs[i] == 0 || n_lhs[i] != 1 || lhs[i][0].kind != VARIABLE) {
-#if	!SILENT
-				printf(_("#%d is not a solved equation.\n"), i + 1);
-#endif
+				error(_("Not a solved equation."));
 				continue;
 			}
-			in_calc_cmd = true;
+			approximate_roots = true;
+			simp_i(rhs[i], &n_rhs[i]);
+			approximate_roots = false;
 			if (int_flag) {
-				elim_loop(rhs[i], &n_rhs[i]);
 				uf_repeat_always(rhs[i], &n_rhs[i]);
 			}
 			group_sub(i);
-			in_calc_cmd = false;
 			if (int_flag) {
 				if (!int_expr(rhs[i], n_rhs[i])) {
-#if	!SILENT
-					printf(_("#%d is not an integer equation.\n"), i + 1);
-#endif
+					error(_("Not an integer equation."));
 					continue;
 				}
 			}
 			list_c_equation(i, language, int_flag);
+			displayed = true;
 		}
 	}
-	return true;
+	return displayed;
 }
 #endif
 
@@ -1694,9 +1714,10 @@ char	*cp;
 			return false;
 		subst_constants(lhs[i], &n_lhs[i]);
 		subst_constants(rhs[i], &n_rhs[i]);
+		approximate_roots = true;
 		simp_sub(i);
-		return_result(i);
-		return true;
+		approximate_roots = false;
+		return return_result(i);
 	}
 	for (vc = 0; *cp; vc++) {
 		if (strcmp_tospace(cp, "with") == 0) {
@@ -1793,8 +1814,7 @@ do_this:
 	n_rhs[i] = n_trhs;
 	blt(rhs[i], trhs, n_trhs * sizeof(token_type));
 	simp_sub(i);
-	return_result(i);
-	return true;
+	return return_result(i);
 }
 
 /*
@@ -1917,6 +1937,28 @@ char	*cp;
 	return true;
 }
 
+/*
+ * The fraction command.
+ */
+int
+fraction_cmd(cp)
+char	*cp;
+{
+	int	i, j;
+
+	if (!get_range_eol(&cp, &i, &j)) {
+		return false;
+	}
+	for (; i <= j; i++) {
+		if (n_lhs[i]) {
+			frac_side(lhs[i], &n_lhs[i]);
+			frac_side(rhs[i], &n_rhs[i]);
+			return_result(i);
+		}
+	}
+	return true;
+}
+
 #if	!LIBRARY
 /*
  * The quit command.
@@ -2080,6 +2122,7 @@ char	*cp;
 		error(_("No file specified."));
 		return false;
 	}
+#if	!SILENT
 	if (access(cp, 0) == 0) {
 		snprintf(prompt_str, sizeof(prompt_str), _("\"%s\" exists.  Overwrite (Y/N)? "), cp);
 		if (!get_yes_no()) {
@@ -2087,6 +2130,7 @@ char	*cp;
 			return false;
 		}
 	}
+#endif
 	fp = fopen(cp, "w");
 	if (fp == NULL) {
 		error(_("Can't create file."));
@@ -2100,7 +2144,9 @@ char	*cp;
 	if (fclose(fp))
 		rv = false;
 	if (rv) {
+#if	!SILENT
 		printf(_("All equations saved in file: \"%s\".\n"), cp);
+#endif
 	} else {
 		error(_("Error encountered while saving equations."));
 	}
