@@ -60,8 +60,8 @@ long		v;
 
 	if (equation[loc].level < level) {
 /* First differentiate if it is a single variable or constant. */
-/* If it is the specified variable, change it to the constant "1.0", */
-/* otherwise change it to "0.0". */
+/* If it is the specified variable, change it to the constant 1, */
+/* otherwise change it to the constant 0. */
 		if (equation[loc].kind == VARIABLE
 		    && ((v == MATCH_ANY && (equation[loc].token.variable & VAR_MASK) > SIGN)
 		    || equation[loc].token.variable == v)) {
@@ -73,8 +73,7 @@ long		v;
 		}
 		return true;
 	}
-	op = 0;
-	for (oploc = endloc = loc + 1; endloc < *np && equation[endloc].level >= level; endloc += 2) {
+	for (op = 0, oploc = endloc = loc + 1; endloc < *np && equation[endloc].level >= level; endloc += 2) {
 		if (equation[endloc].level == level) {
 			switch (op) {
 			case 0:
@@ -120,9 +119,9 @@ long		v;
 		return true;
 	}
 /* Differentiate PLUS and MINUS operators. */
-/* Use rule: d(u+v) = d(u) + d(v), */
-/* where "d()" is the derivative function, */
-/* "u" and "v" are expressions. */
+/* Use addition rule: d(u+v) = d(u) + d(v), */
+/* where "d()" is the derivative function */
+/* and "u" and "v" are expressions. */
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].kind != OPERATOR) {
 			if (!d_recurse(equation, np, i, level + 1, v))
@@ -137,7 +136,7 @@ long		v;
 	return true;
 d_times:
 /* Differentiate TIMES operator. */
-/* Use rule: d(u*v) = u*d(v) + v*d(u). */
+/* Use product rule: d(u*v) = u*d(v) + v*d(u). */
 	if (*np + 1 + (endloc - loc) > n_tokens) {
 		error_huge();
 	}
@@ -153,7 +152,7 @@ d_times:
 	return(d_recurse(equation, np, loc, level + 2, v));
 d_divide:
 /* Differentiate DIVIDE operator. */
-/* Use rule: d(u/v) = (v*d(u) - u*d(v))/v^2. */
+/* Use quotient rule: d(u/v) = (v*d(u) - u*d(v))/v^2. */
 	if (*np + 3 + (endloc - loc) + (endloc - oploc) > n_tokens) {
 		error_huge();
 	}
@@ -186,7 +185,7 @@ d_divide:
 	return(d_recurse(equation, np, loc, level + 3, v));
 d_power:
 /* Differentiate POWER operator. */
-/* Since we don't have logarithms, just do what we can without them. */
+/* Since we don't have symbolic logarithms, do all we can without them. */
 	for (i = oploc; i < endloc; i++) {
 		if (equation[i].kind == VARIABLE
 		    && ((v == MATCH_ANY && (equation[i].token.variable & VAR_MASK) > SIGN)
@@ -289,7 +288,7 @@ derivative_cmd(cp)
 char	*cp;
 {
 	long		v = 0;
-	int		i, j;
+	int		i;
 	long		l1, order = 1;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
@@ -309,6 +308,7 @@ char	*cp;
 		dest = lhs[i];
 		np = &n_lhs[i];
 	}
+/* parse the command line or prompt: */
 	if (*cp) {
 		if (is_all(cp)) {
 			cp = skip_param(cp);
@@ -329,21 +329,9 @@ char	*cp;
 			return false;
 		}
 	}
-	if (v == 0) {
-		for (j = 0; j < *nps; j += 2) {
-			if (source[j].kind == VARIABLE) {
-				if ((source[j].token.variable & VAR_MASK) <= SIGN)
-					continue;
-				if (v) {
-					if (v != source[j].token.variable) {
-						v = 0;
-						break;
-					}
-				} else {
-					v = source[j].token.variable;
-				}
-			}
-		}
+	if (no_vars(source, *nps, &v)) {
+		error(_("Current expression contains no variables, result would be zero."));
+		return false;
 	}
 	if (v == 0) {
 		if (!prompt_var(&v)) {
@@ -351,7 +339,7 @@ char	*cp;
 		}
 	}
 	if (v != MATCH_ANY && !found_var(source, *nps, v)) {
-		error(_("Variable not found.  Result would be zero."));
+		error(_("Variable not found, result would be zero."));
 		return false;
 	}
 #if	!SILENT
@@ -365,6 +353,7 @@ char	*cp;
 #endif
 	blt(dest, source, *nps * sizeof(token_type));
 	n1 = *nps;
+/* do the actual differentiating and simplifying: */
 	for (l1 = 0; l1 < order; l1++) {
 		if (!differentiate(dest, &n1, v)) {
 			error(_("Differentiation failed."));
@@ -378,8 +367,7 @@ char	*cp;
 	}
 	*np = n1;
 	cur_equation = i;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
 /*
@@ -389,7 +377,7 @@ int
 extrema_cmd(cp)
 char	*cp;
 {
-	int		i, j;
+	int		i;
 	long		v = 0;
 	token_type	want;
 	token_type	*source;
@@ -418,21 +406,9 @@ char	*cp;
 			return false;
 		}
 	}
-	if (v == 0) {
-		for (j = 0; j < n; j += 2) {
-			if (source[j].kind == VARIABLE) {
-				if ((source[j].token.variable & VAR_MASK) <= SIGN)
-					continue;
-				if (v) {
-					if (v != source[j].token.variable) {
-						v = 0;
-						break;
-					}
-				} else {
-					v = source[j].token.variable;
-				}
-			}
-		}
+	if (no_vars(source, n, &v)) {
+		error(_("Current expression contains no variables, there are no extrema."));
+		return false;
 	}
 	if (v == 0) {
 		if (!prompt_var(&v)) {
@@ -445,19 +421,22 @@ char	*cp;
 	}
 	i = next_espace();
 	blt(rhs[i], source, n * sizeof(token_type));
-	n_rhs[i] = n;
-	if (!differentiate(rhs[i], &n_rhs[i], v)) {
+/* take the derivative with respect to the specified variable and simplify: */
+	if (!differentiate(rhs[i], &n, v)) {
 		error(_("Differentiation failed."));
 		return false;
 	}
-	simpa_side(rhs[i], &n_rhs[i], true);
-	if (!found_var(rhs[i], n_rhs[i], v)) {
+	simpa_side(rhs[i], &n, true);
+	if (!found_var(rhs[i], n, v)) {
 		error(_("There are no extrema for this equation."));
 		return false;
 	}
+	n_rhs[i] = n;
+/* set equal to zero: */
 	n_lhs[i] = 1;
 	lhs[i][0] = zero_token;
 	cur_equation = i;
+/* lastly, solve for the specified variable and simplify: */
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
@@ -466,8 +445,7 @@ char	*cp;
 		return false;
 	}
 	simpa_side(rhs[i], &n_rhs[i], false);
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
 /*
@@ -527,25 +505,14 @@ char	*cp;
 		}
 		cp = cp1;
 	}
-	if (v == 0) {
-		for (j = 0; j < *nps; j += 2) {
-			if (source[j].kind == VARIABLE) {
-				if ((source[j].token.variable & VAR_MASK) <= SIGN)
-					continue;
-				if (v) {
-					if (v != source[j].token.variable) {
-						v = 0;
-						break;
-					}
-				} else {
-					v = source[j].token.variable;
-				}
-			}
-		}
+	if (no_vars(source, *nps, &v)) {
+		error(_("Current expression contains no variables."));
+		return false;
 	}
 	if (v == 0) {
-		if (!prompt_var(&v))
+		if (!prompt_var(&v)) {
 			return false;
+		}
 	}
 	if (!found_var(source, *nps, v)) {
 		error(_("Variable not found."));
@@ -690,8 +657,7 @@ loop_again:
 	}
 	*np = n1;
 	cur_equation = i;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
 /*
@@ -714,12 +680,13 @@ char	*cp;
 		return false;
 	}
 	if (n_rhs[cur_equation] == 0) {
+/* make expression into equation: */
 		blt(rhs[cur_equation], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_rhs[cur_equation] = n_lhs[cur_equation];
 		n_lhs[cur_equation] = 1;
 		lhs[cur_equation][0].level = 1;
 		lhs[cur_equation][0].kind = VARIABLE;
-		lhs[cur_equation][0].token.variable = V_ANSWER;
+		parse_var(&lhs[cur_equation][0].token.variable, "answer");
 	}
 	if (!solved_equation(cur_equation)) {
 		error(_("The current equation is not solved for a variable."));
@@ -727,12 +694,18 @@ char	*cp;
 	}
 	solved_v = lhs[cur_equation][0];
 	i = next_espace();
+/* parse the command line or prompt: */
 	if (*cp) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
 		}
-	} else {
+	}
+	if (no_vars(rhs[cur_equation], n_rhs[cur_equation], &v)) {
+		error(_("Current expression contains no variables."));
+		return false;
+	}
+	if (v == 0) {
 		if (!prompt_var(&v)) {
 			return false;
 		}
@@ -750,7 +723,8 @@ char	*cp;
 			return false;
 		}
 	} else {
-		my_strlcpy(prompt_str, _("goes to: "), sizeof(prompt_str));
+		list_var(v, 0);
+		snprintf(prompt_str, sizeof(prompt_str), _("as (%s) goes to: "), var_str);
 		if (!get_expr(lhs[i], &nl)) {
 			return false;
 		}
@@ -759,6 +733,7 @@ char	*cp;
 		return false;
 	simp_loop(lhs[i], &nl);
 	infinity_flag = exp_contains_infinity(lhs[i], nl);
+/* fully simplify the current equation and group denominators together: */
 	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 	group_proc(rhs[cur_equation], &n_rhs[cur_equation]);
 	found_den = false;
@@ -773,6 +748,7 @@ char	*cp;
 			for (k = j + 2;; k += 2) {
 				if (rhs[cur_equation][k-1].kind == VARIABLE
 				    && rhs[cur_equation][k-1].token.variable == v) {
+/* the limit variable was found in a denominator */
 					den = true;
 					found_den = true;
 				}
@@ -782,6 +758,7 @@ char	*cp;
 				switch (rhs[cur_equation][k].token.operatr) {
 				case DIVIDE:
 				case MODULUS:
+/* a denominator contains a fraction */
 					complex_den = true;
 					break;
 				}
@@ -789,6 +766,7 @@ char	*cp;
 			for (k = j - 1; k >= 0 && rhs[cur_equation][k].level >= level; k--) {
 				if (rhs[cur_equation][k].kind == VARIABLE
 				    && rhs[cur_equation][k].token.variable == v) {
+/* the limit variable was found in a numerator */
 					num = true;
 				}
 			}
@@ -799,12 +777,14 @@ char	*cp;
 		}
 	}
 	if (!found_den || (infinity_flag && !complex_den)) {
+/* in this case, just substitute the limit variable with the limit expression and simplify: */
 		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], lhs[i], nl, v);
 		simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
-		return_result(cur_equation);
-		return true;
+		return return_result(cur_equation);
 	}
 	if (infinity_flag) {
+/* To take the limit to infinity, */
+/* replace infinity with zero and replace the limit variable with its reciprocal: */
 		nl = 1;
 		lhs[i][0] = zero_token;
 		tlhs[0] = one_token;
@@ -817,6 +797,7 @@ char	*cp;
 		n_tlhs = 3;
 		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tlhs, n_tlhs, v);
 	}
+/* General limit taking, solve for the limit variable: */
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
@@ -824,18 +805,21 @@ char	*cp;
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
+/* replace the limit variable (LHS) with the limit expression: */
 	blt(lhs[cur_equation], lhs[i], nl * sizeof(token_type));
 	n_lhs[cur_equation] = nl;
+/* symbolically simplify the RHS: */
 	symb_flag = true;
 	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 	symb_flag = false;
+/* solve back for the original variable: */
 	if (solve_sub(&solved_v, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) <= 0) {
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
+/* symbolically simplify before returning the result: */
 	symb_flag = true;
 	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 	symb_flag = false;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
