@@ -18,8 +18,7 @@ static int	fpower_recurse();
 static int	fpower_sub();
 
 /*
- * Factor divides only.
- * (a/c + b/c) -> ((a+b)/c).
+ * Factor divides only: (a/c + b/c) -> ((a+b)/c).
  *
  * Return true if equation side was modified.
  */
@@ -30,13 +29,12 @@ int		*np;
 long		v;
 double		d;
 {
-	return fplus_recurse(equation, np, 0, 1, v, d, 2);
+	return fplus_recurse(equation, np, 0, 1, v, d, false, true);
 }
 
 /*
  * Take care of subtraction and addition of the same expression
- * multiplied by constants.
- * (2*a + 3*a - a) -> (4*a).
+ * multiplied by constants: (2*a + 3*a - a) -> (4*a).
  *
  * Return true if equation side was modified.
  */
@@ -45,17 +43,18 @@ subtract_itself(equation, np)
 token_type	*equation;
 int		*np;
 {
-	return fplus_recurse(equation, np, 0, 1, 0L, 0.0, 1);
+	return fplus_recurse(equation, np, 0, 1, 0L, 0.0, true, false);
 }
 
 /*
  * Factor equation side: (a*c + b*c) -> (c*(a + b)).
  *
  * If "v" and "d" equals 0, factor anything,
- * including identical bases raised to powers: (x^2 + x) -> (x*(x + 1)).
+ * including identical bases raised to powers (Horner factoring): (x^2 + x) -> (x*(x + 1)).
  * If "d" equals 1.0, only factor identical bases raised to the power of a constant.
  *
- * If "v" is a variable, only factor expressions containing that variable.
+ * If "v" is a variable, only factor expressions containing that variable,
+ * with no Horner factoring.
  * If "d" is not equal to 0.0 or 1.0, factor only expressions containing "v" raised
  * to the power of "d".
  *
@@ -68,7 +67,7 @@ int		*np;		/* pointer to length of equation side */
 long		v;		/* variable */
 double		d;		/* control exponent */
 {
-	return fplus_recurse(equation, np, 0, 1, v, d, 0);
+	return fplus_recurse(equation, np, 0, 1, v, d, false, false);
 }
 
 /*
@@ -78,12 +77,13 @@ double		d;		/* control exponent */
  * Return true if the expression was modified.
  */
 static int
-fplus_recurse(equation, np, loc, level, v, d, factor_code)
+fplus_recurse(equation, np, loc, level, v, d, whole_flag, div_only)
 token_type	*equation;
 int		*np, loc, level;
 long		v;
 double		d;
-int		factor_code;
+int		whole_flag;	/* factor only whole expressions multiplied by a constant */
+int		div_only;	/* factor only divides */
 {
 	int	modified = false;
 	int	i, j, k;
@@ -115,7 +115,7 @@ f_again:
 						break;
 				}
 				len2 = k - j;
-				if (fplus_sub(equation, np, loc, i, len1, j, len2, level + 1, factor_code == 1, factor_code == 2, v, d)) {
+				if (fplus_sub(equation, np, loc, i, len1, j, len2, level + 1, v, d, whole_flag, div_only)) {
 					modified = true;
 					goto f_again;
 				}
@@ -129,7 +129,7 @@ f_again:
 		return true;
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
-			modified |= fplus_recurse(equation, np, i, level + 1, v, d, factor_code);
+			modified |= fplus_recurse(equation, np, i, level + 1, v, d, whole_flag, div_only);
 			i++;
 			for (; i < *np && equation[i].level > level; i += 2)
 				;
@@ -146,16 +146,16 @@ f_again:
  * Return true if a transformation was made.
  */
 static int
-fplus_sub(equation, np, loc, i1, n1, i2, n2, level, whole_flag, div_only, v, d)
+fplus_sub(equation, np, loc, i1, n1, i2, n2, level, v, d, whole_flag, div_only)
 token_type	*equation;	/* the entire expression */
 int		*np;		/* pointer to length of the entire expression */
 int		loc;		/* index into the beginning of this additive subexpression */
 int		i1, n1, i2, n2;
 int		level;
-int		whole_flag;	/* factor only whole expressions multiplied by a constant */
-int		div_only;	/* factor only divides */
 long		v;
 double		d;
+int		whole_flag;	/* factor only whole expressions multiplied by a constant */
+int		div_only;	/* factor only divides */
 {
 	int	e1, e2;
 	int	op1, op2;
@@ -169,7 +169,7 @@ double		d;
 	int	same_flag;
 	double	save_k1, save_k2;
 	double	save_d1, save_d2;
-	double	power;
+	double	power;		/* for constant power horner factoring */
 	double	d1, d2;
 
 	e1 = i1 + n1;
@@ -300,8 +300,8 @@ f_inner:
 	}
 	if (same_flag) {
 		/* do the factor transformation */
-		power = 1.0;
-more_power:
+		power = 1.0;	/* not doing Horner factoring */
+horner_factor:
 		if (sop1 == DIVIDE) {
 			scratch[0].level = level;
 			scratch[0].kind = CONSTANT;
@@ -442,7 +442,7 @@ end_mess:
 	if (whole_flag)
 		return false;
 	if (v || div_only)
-		goto f_inner;
+		goto f_inner;	/* don't do Horner factoring */
 	if (b1 == i1 && i == e1)
 		k = level;
 	else
@@ -457,8 +457,9 @@ end_mess:
 				save_d1 = equation[l+1].token.constant;
 				if (save_d1 <= 0.0)
 					goto f_inner;
-			} else
+			} else {
 				save_d1 = -1.0;
+			}
 			ai = l;
 			break;
 		}
@@ -477,8 +478,9 @@ end_mess:
 				save_d2 = equation[l+1].token.constant;
 				if (save_d2 <= 0.0)
 					goto f_inner;
-			} else
+			} else {
 				save_d2 = -1.0;
+			}
 			aj = l;
 			break;
 		}
@@ -499,14 +501,14 @@ end_mess:
 				power = min(save_d1, save_d2);
 				if (!diff_sign && (fmod(power, 1.0) != 0.0)) {
 					if (fmod(max(save_d1, save_d2) - power, 1.0) == 0.0) {
-						goto more_power;
+						goto horner_factor;
 					}
 				}
 			}
 			if (power < 1.0)
 				goto f_inner;
 			modf(power, &power);
-			goto more_power;
+			goto horner_factor;
 		}
 		d1 = i - ai;
 		d2 = j - aj;
@@ -548,7 +550,7 @@ end_mess:
 
 /*
  * Factor transformation for a more general pair of subexpressions added together
- * with a common base.
+ * with a common base and any exponent.
  */
 static int
 big_fplus(equation, level, diff_sign, sop1, op1, op2, i1, i2, b1, b2, ai, aj, i, j, e1, e2)
