@@ -44,7 +44,8 @@ static com_type com_list[] = {
 {	"eliminate",	NULL,			eliminate_cmd,	"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
 {	"extrema",	"extremum",		extrema_cmd,	"[variable]",						"Find the minimums and maximums of the current expression." },
 {	"factor",	NULL,			factor_cmd,	"[\"number\" [integers]] or [equation-number-range] [variables]",	"Trivially factor integers or stored expressions." },
-{	"flist",	"fraction",		flist_cmd,	"[\"factor\"] [equation-number-range]",			"Display stored expressions in multi-line fraction format." },
+{	"flist",	"display",		flist_cmd,	"[\"factor\"] [equation-number-range]",			"Display stored expressions in multi-line fraction format." },
+{	"fraction",	NULL,			fraction_cmd,	"[equation-number-range]",				"Convert expressions with any algebraic fractions into a single fraction." },
 {	"help",		NULL,			help_cmd,	"[topic or command-name]",				"Short, built-in help." },
 {	"imaginary",	NULL,			imaginary_cmd,	"[variable]",						"Copy the imaginary part of an expression (see the \"real\" command)." },
 {	"integrate",	"integral",		integrate_cmd,	"[\"definite\"] variable [order]",			"Symbolically integrate polynomials, order times." },
@@ -52,7 +53,7 @@ static com_type com_list[] = {
 {	"limit",	NULL,			limit_cmd,	"variable expression",					"Take the limit of the current expression as variable goes to expression." },
 {	"list",		NULL,			list_cmd,	"[\"export\" or \"maxima\"] [equation-number-range]",	"Display stored expressions in single line format." },
 #if	!LIBRARY
-{	"nintegrate",	NULL,			nintegrate_cmd,	"[\"trapezoidal\"] variable [partitions]",		"Approximate the definite integral using Simpson's rule."},
+{	"nintegrate",	NULL,			nintegrate_cmd,	"[\"trapezoid\"] variable [partitions]",		"Approximate the definite integral using Simpson's rule."},
 {	"optimize",	NULL,			optimize_cmd,	"[equation-number-range]",				"Split up an equation into smaller multiple equations." },
 #endif
 #if	!LIBRARY
@@ -69,7 +70,7 @@ static com_type com_list[] = {
 {	"read",		NULL,			read_cmd,	"file-name",						"Read in a text file as if it was typed in." },
 #endif
 {	"real",		NULL,			real_cmd,	"[variable]",						"Copy the real part of an expression (see the \"imaginary\" command)." },
-{	"replace",	NULL,			replace_cmd,	"[\"constants\"] [variables [\"with\" expression]]",	"Substitute variables in the current equation with expressions." },
+{	"replace",	NULL,			replace_cmd,	"[\"constants\"] or [variables [\"with\" expression]]",	"Substitute variables in the current equation with expressions." },
 #if	!LIBRARY
 {	"roots",	NULL,			roots_cmd,	"root real-part imaginary-part",			"Display all the roots of a complex number." },
 #endif
@@ -85,9 +86,7 @@ static com_type com_list[] = {
 #endif
 {	"taylor",	NULL,			taylor_cmd,	"[variable] [order] [point]",				"Compute the Taylor series expansion of the current expression." },
 {	"unfactor",	"expand",		unfactor_cmd,	"[\"fully\"] [equation-number-range]",			"Algebraically expand (multiply out) stored expressions." },
-#if	!LIBRARY
 {	"version",	NULL,			version_cmd,	"",							"Display version number and compile flags used." }
-#endif
 };
 
 /*
@@ -106,11 +105,13 @@ char	*cp;
 	if ((cp = parse_equation(n, cp)) != NULL) {
 		if (n_lhs[n] == 0 && n_rhs[n] == 0)
 			return true;
-		if (n_lhs[n]) {
-			cur_equation = n;
-			return_result(cur_equation);
-			return true;
+		if (n_lhs[n] == 0) {
+			/* RHS expression only, set equal to zero */
+			n_lhs[n] = 1;
+			lhs[n][0] = zero_token;
 		}
+		cur_equation = n;
+		return return_result(cur_equation);
 	}
 	n_lhs[n] = 0;
 	n_rhs[n] = 0;
@@ -130,23 +131,32 @@ process_parse(n, cp)
 int	n;
 char	*cp;
 {
+#if	!LIBRARY
 	int	i;
+#endif
 
 	if ((cp = parse_equation(n, cp)) != NULL) {
 		if (n_lhs[n] == 0 && n_rhs[n] == 0)
 			return true;
 		if (n_lhs[n] == 0 || n_rhs[n] == 0) {
-			if ((n_lhs[n] == 1 && ((lhs[n][0].kind == CONSTANT && lhs[n][0].token.constant == 0.0)
-			    || (lhs[n][0].kind == VARIABLE && (lhs[n][0].token.variable & VAR_MASK) > SIGN)))
-			    || n_rhs[n]) {
-				if (solve(n, cur_equation)) {
-					return_result(cur_equation);
-					return true;
+			if (autosolve) {
+				if ((n_lhs[n] == 1 && ((lhs[n][0].kind == CONSTANT && lhs[n][0].token.constant == 0.0)
+				    || (lhs[n][0].kind == VARIABLE && (lhs[n][0].token.variable & VAR_MASK) > SIGN)))
+				    || (n_rhs[n] == 1 && ((rhs[n][0].kind == CONSTANT && rhs[n][0].token.constant == 0.0)
+				    || (rhs[n][0].kind == VARIABLE && (rhs[n][0].token.variable & VAR_MASK) > SIGN)))) {
+					if (solve(n, cur_equation)) {
+						return return_result(cur_equation);
+					}
+					return false;
 				}
-				return false;
 			}
-			if (n_lhs[n] == 1 && lhs[n][0].kind == CONSTANT
-			    && fmod(lhs[n][0].token.constant, 1.0) == 0.0
+			if (n_rhs[n]) {
+				/* RHS expression only, set equal to zero */
+				n_lhs[n] = 1;
+				lhs[n][0] = zero_token;
+				goto return_ok;
+			}
+			if (n_lhs[n] == 1 && lhs[n][0].kind == CONSTANT && fmod(lhs[n][0].token.constant, 1.0) == 0.0
 			    && lhs[n][0].token.constant > 0.0 && lhs[n][0].token.constant <= n_equations) {
 				/* easy selecting of equation spaces */
 				n_lhs[n] = 0;
@@ -165,7 +175,7 @@ char	*cp;
 			n_rhs[n] = n_lhs[n];
 			lhs[n][0].level = 1;
 			lhs[n][0].kind = VARIABLE;
-			lhs[n][0].token.variable = V_ANSWER;
+			parse_var(&lhs[n][0].token.variable, "answer");
 			n_lhs[n] = 1;
 			i = cur_equation;
 			cur_equation = n;
@@ -178,8 +188,7 @@ char	*cp;
 		}
 return_ok:
 		cur_equation = n;
-		return_result(cur_equation);
-		return true;
+		return return_result(cur_equation);
 	}
 	n_lhs[n] = 0;
 	n_rhs[n] = 0;
@@ -240,7 +249,7 @@ char	*cp;
 			return parse(i, cp);
 		}
 		cur_equation = i;
-		return_result(cur_equation);
+		return return_result(cur_equation);
 	}
 #if	!SECURE
 	/* handle shell escape */
@@ -253,6 +262,7 @@ char	*cp;
 		if (cp1) {
 			return(!shell_out(cp1));
 		}
+		error(_("SHELL environment variable not set."));
 		return false;
 	}
 #endif
@@ -353,19 +363,18 @@ char	*cp;
 #endif
 
 /*
- * Parse a variable name, with error messages and space character skipping.
+ * Parse a variable name with before and after space character skipping.
  *
  * Return new position in string or NULL if error.
  */
 char	*
 parse_var2(vp, cp)
-long	*vp;	/* pointer to result */
+long	*vp;	/* pointer to returned variable in Mathomatic internal format */
 char	*cp;	/* pointer to variable name string */
 {
 	cp = skip_space(cp);
 	cp = parse_var(vp, cp);
 	if (cp == NULL) {
-		error(_("Invalid variable."));
 		return NULL;
 	}
 	return skip_space(cp);
@@ -414,10 +423,11 @@ char	*cp;
 			P("    Area = b*h/2");
 			P("Rectangle of length \"l\" and width \"w\":");
 			P("    Area = l*w                    Perimeter = 2*l + 2*w");
-			P("Trapezoid of height \"h\" and parallel sides \"a\" and \"b\":");
-			P("    Area = h*(a+b)/2");
+			P("Trapezoid of parallel sides \"a\" and \"b\", and \"d\" distance between them:");
+			P("    Area = d*(a + b)/2");
 			P("Circle of radius \"r\":");
 			P("    Area = pi*r^2                 Perimeter = 2*pi*r");
+			P("    Area = Perimeter*r/2");
 			P("Rectangular solid of length \"l\", width \"w\", and height \"h\":");
 			P("    Volume = l*w*h                Surface area = 2*l*w + 2*l*h + 2*w*h");
 			P("Sphere of radius \"r\":");
@@ -427,8 +437,8 @@ char	*cp;
 			P("Right circular cone of radius \"r\" and height \"h\":");
 			P("    Volume = pi*r^2*h/3");
 			P("    Base surface area = pi*r^2    Side surface area = pi*r*(r^2 + h^2)^.5\n");
-			P("Convex polygon of \"n\" sides sum of all interior angles formula:");
-			P("    Sum = (n-2)*180 degrees");
+			P("Convex polygon of \"n\" sides, sum of all interior angles formula:");
+			P("    Sum = (n - 2)*180 degrees     Sum = (n - 2)*pi radians");
 			return true;
 		}
 		if (strncasecmp(cp, "expressions", cp1 - cp) == 0 || strncasecmp(cp, "equations", cp1 - cp) == 0) {
@@ -440,12 +450,13 @@ char	*cp;
 			P("    * multiply        / divide          % modulus");
 			P("    + add             - subtract\n");
 			P("Operators in the same precedence level are evaluated left to right.\n");
-			P("Variables consist of any combination of letters, digits, underscores (_),");
-			P("and backslashes (\\).  Predefined constants and variables:\n");
-			P("    e# - the universal constant e (2.71828182846...).");
-			P("    pi - the universal constant pi (3.14159265359...).");
+			P("Variables consist of any combination of letters, digits, and underscores (_).");
+			P("Constants always start with a digit (0..9) or a period.");
+			P("Predefined constants and variables follow:\n");
+			P("    e# - the universal constant e (2.7182818284...).");
+			P("    pi - the universal constant pi (3.1415926535...).");
 			P("    i# - imaginary number (square root of -1).");
-			P("    sign, sign1, sign2, sign3, ... - may be +1 or -1.");
+			P("    sign, sign1, sign2, sign3, ... - may only be +1 or -1.");
 			P("    integer - may be any integer.");
 			P("    inf - infinity constant.\n");
 			P("Absolute value notation \"|x|\" and \"+/-\" are understood.");
@@ -474,7 +485,7 @@ char	*cp;
 	P("Type \"help usage\" for syntax of all commands.");
 	P("Type \"help expressions\" for help with entering expressions.");
 	P("Type \"help geometry\" for some commonly used geometric formulas.");
-	P("\"help\" followed by a command name will give info on a command.\n");
+	P("\"help\" or \"?\" followed by a command name will give info on a command.\n");
 	P("Available commands:");
 	for (i = 0; i < ARR_CNT(com_list); i++) {
 		if ((i % 5) == 0)
