@@ -39,6 +39,7 @@ reset_attr()
 			printf("\033[0m");
 		}
 	}
+	fflush(NULL);	/* flush all output */
 }
 
 /*
@@ -78,7 +79,7 @@ default_color()
 }
 
 /*
- * Display the expression or equation stored in equation space "n" in single line format.
+ * Display the expression or equation stored in equation space "n" in single-line format.
  * The equation space is not modified.
  *
  * Return true if successful.
@@ -125,13 +126,15 @@ int	n;	/* equation space number */
 {
 	if (n_lhs[n] <= 0)
 		return false;
-	if (groupall) {
+	if (display2d) {
+		/* display in fraction format */
 		group_sub(n);
 		if (factor_int_flag) {
 			factor_int_sub(n);
 		}
 		return flist_sub(n);
 	} else {
+		/* display in single-line format */
 		if (factor_int_flag) {
 			factor_int_sub(n);
 		}
@@ -208,15 +211,15 @@ int	lang_code;	/* language code */
 		break;
 	case V_E:
 		switch (lang_code) {
-		case -2:
-			cp = "e";
-			break;
 		case -1:
 			cp = "%e";
 			break;
 		case 0:
 			cp = "e#";
 			break;
+		case 1:
+			cp ="M_E";
+			break;
 		case 2:
 			cp = "Math.E";
 			break;
@@ -224,7 +227,7 @@ int	lang_code;	/* language code */
 			cp = "math.e";
 			break;
 		default:
-			cp = "E";
+			cp = "e";
 			break;
 		}
 		break;
@@ -234,7 +237,7 @@ int	lang_code;	/* language code */
 			cp = "%pi";
 			break;
 		case 1:
-			cp = "PI";
+			cp = "M_PI";
 			break;
 		case 2:
 			cp = "Math.PI";
@@ -250,15 +253,6 @@ int	lang_code;	/* language code */
 	case MATCH_ANY:
 		cp = "all";
 		break;
-	case V_ANSWER:
-		cp = "answer";
-		break;
-	case V_TEMP:
-		cp = "temp";
-		break;
-	case V_INTEGER:
-		cp = "integer";
-		break;
 	default:
 		j = (v & VAR_MASK) - VAR_OFFSET;
 		if (j >= 0) {
@@ -335,7 +329,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
 			} else if (finance_option) {
 				len += fprintf(gfp, "%.2f", equation[i].token.constant);
 			} else {
-				len += fprintf(gfp, "%.12g", equation[i].token.constant);
+				len += fprintf(gfp, "%.*g", precision, equation[i].token.constant);
 			}
 			break;
 		case VARIABLE:
@@ -401,8 +395,10 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	}
 	len += 2;
 	cp = (char *) malloc(len);
-	if (cp == NULL)
+	if (cp == NULL) {
+		error(_("Can't malloc()."));
 		return NULL;
+	}
 	list_string_sub(lhs[n], n_lhs[n], cp, export_flag);
 	if (n_rhs[n]) {
 		strcat(cp, " = ");
@@ -432,8 +428,10 @@ int		export_flag;
 	len = list_string_sub(equation, n, NULL, export_flag);
 	len++;
 	cp = (char *) malloc(len);
-	if (cp == NULL)
+	if (cp == NULL) {
+		error(_("Can't malloc()."));
 		return NULL;
+	}
 	list_string_sub(equation, n, cp, export_flag);
 	return cp;
 }
@@ -492,7 +490,7 @@ int		export_flag;
 			} else if (finance_option) {
 				len += snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
 			} else {
-				len += snprintf(buf, sizeof(buf), "%.12g", equation[i].token.constant);
+				len += snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
 			}
 			if (string) {
 				strcat(string, buf);
@@ -553,11 +551,10 @@ token_type	*equation;	/* expression pointer */
 int		n;		/* length of expression */
 {
 	int	i;
-	double	d;
 
 	for (i = 0; i < n; i++) {
 		if ((equation[i].kind == CONSTANT && fmod(equation[i].token.constant, 1.0))
-		    || (equation[i].kind == VARIABLE && var_is_const(equation[i].token.variable, &d))) {
+		    || (equation[i].kind == VARIABLE && equation[i].token.variable <= IMAGINARY)) {
 			return false;
 		}
 	}
@@ -660,7 +657,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 			if (int_flag) {
 				snprintf(buf, sizeof(buf), "%.0f", equation[i].token.constant);
 			} else {
-				snprintf(buf, sizeof(buf), "%#.16g", equation[i].token.constant);
+				snprintf(buf, sizeof(buf), "%#.*g", MAX_PRECISION, equation[i].token.constant);
 				j = strlen(buf) - 1;
 				for (; j >= 0; j--) {
 					if (buf[j] == '0')
@@ -966,7 +963,7 @@ check_again:
 			if (finance_option) {
 				len += snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
 			} else {
-				len += snprintf(buf, sizeof(buf), "%.12g", equation[i].token.constant);
+				len += snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
 			}
 			if (oflag)
 				fprintf(gfp, "%s", buf);
