@@ -84,25 +84,30 @@ char	**argv;
 	gfp = stdout;
 
 	/* process command line options */
-	while ((i = getopt(argc, argv, "qtchuvm:")) >= 0) {
+	while ((i = getopt(argc, argv, "qrtchuvm:")) >= 0) {
 		switch (i) {
 		case 'c':
 			coption = true;
 			break;
 		case 'h':
+			readline_enabled = false;
 			html_flag = true;
 			break;
 		case 'm':
 			multiplier = atof(optarg);
-			if (multiplier <= 0.0 || (n_tokens = (int) (multiplier * DEFAULT_N_TOKENS)) <= 0) {
-				fprintf(stderr, _("Invalid memory size multiplier specified!\n"));
+			if (multiplier <= 0.0 || (n_tokens = (int) (multiplier * DEFAULT_N_TOKENS)) < 100) {
+				fprintf(stderr, _("Invalid memory size multiplier specified.\n"));
 				usage();
 			}
 			break;
 		case 'q':
 			quiet_mode = true;
 			break;
+		case 'r':
+			readline_enabled = false;
+			break;
 		case 't':
+			readline_enabled = false;
 			test_mode = true;
 			break;
 		case 'u':
@@ -173,7 +178,7 @@ char	**argv;
 		}
 	} else {
 		if (!set_signals()) {
-			fprintf(stderr, _("Signal(2) setting failed.\n"));
+			fprintf(stderr, _("signal(2) setting failed.\n"));
 		}
 		if (!f_to_fraction(0.5, &numerator, &denominator)
 		    || !f_to_fraction(1.0/3.0, &numerator, &denominator)) {
@@ -209,9 +214,10 @@ usage()
 	fprintf(stderr, _("Usage: %s [ options ] [ input_files ]\n\n"), prog_name);
 	fprintf(stderr, _("Options:\n"));
 	fprintf(stderr, _("  -c               Toggle color mode.\n"));
-	fprintf(stderr, _("  -h               Enable HTML mode.\n"));
+	fprintf(stderr, _("  -h               Enable HTML mode and disable readline.\n"));
 	fprintf(stderr, _("  -m number        Specify a memory size multiplier.\n"));
 	fprintf(stderr, _("  -q               Quiet mode (don't display prompts).\n"));
+	fprintf(stderr, _("  -r               Disable readline.\n"));
 	fprintf(stderr, _("  -t               Used when testing.\n"));
 	fprintf(stderr, _("  -u               Unbuffered output.\n"));
 	fprintf(stderr, _("  -v               Display version number and compile flags used.\n"));
