@@ -6,9 +6,9 @@
 
 #include "includes.h"
 
-#define	INFINITY_NAME	"inf"
+#define	INFINITY_NAME	"inf"	/* set this to the name of the infinity constant as displayed by printf(3) */
 
-int	point_flag;	/* point to error if true */
+int	point_flag;		/* point to location of parse error if true */
 
 /*
  * Return true if character is a valid starting variable character.
@@ -143,13 +143,17 @@ char		*cp;		/* string to parse */
 			if (operand) {
 				goto syntax_error;
 			}
+			if (cp[1] == '!') {
+				put_up_arrow((int) (cp - cp_start), _("Multifactorial not supported yet."));
+				return(NULL);
+			}
 			equation[n].level = cur_level;
 			equation[n].kind = OPERATOR;
 			equation[n].token.operatr = FACTORIAL;
 			n++;
 			equation[n].level = cur_level;
 			equation[n].kind = CONSTANT;
-			equation[n].token.constant = 0.0;
+			equation[n].token.constant = 1.0;
 			n++;
 			operand = true;
 			break;
@@ -277,16 +281,16 @@ parse_power:
 				equation[n].token.operatr = TIMES;
 				n++;
 			}
-			if (strncmp(cp, INFINITY_NAME, 3) == 0 && !isvarchar(cp[3])) {
+			if (strncmp(cp, INFINITY_NAME, strlen(INFINITY_NAME)) == 0
+			    && !isvarchar(cp[strlen(INFINITY_NAME)])) {
 				equation[n].kind = CONSTANT;
 				equation[n].token.constant = HUGE_VAL;	/* the infinity constant */
-				cp += 3;
+				cp += strlen(INFINITY_NAME);
 			} else {
 				equation[n].kind = VARIABLE;
 				cp1 = cp;
 				cp = parse_var(&equation[n].token.variable, cp);
 				if (cp == NULL) {
-					put_up_arrow((int) (cp1 - cp_start), _("Invalid variable."));
 					return(NULL);
 				}
 			}
@@ -321,10 +325,13 @@ syntax_error:
 }
 
 /*
- * Parse variable name pointed to by "cp".
- * Variable name is converted to Mathomatic format and stored in "*vp".
+ * Parse variable name string pointed to by "cp".
+ * The variable name is converted to Mathomatic format and stored in "*vp".
  *
- * Return new string position, or NULL on failure.
+ * If the variable is not special and never existed before, it is created.
+ *
+ * Return new string position if successful.
+ * Display error message and return NULL on failure.
  */
 char	*
 parse_var(vp, cp)
@@ -338,15 +345,14 @@ char	*cp;
 	int	len;
 	int	(*strcmpfunc)();
 
+	*vp = V_NULL;
 	if (case_sensitive_flag) {
 		strcmpfunc = strcmp;
 	} else {
 		strcmpfunc = strcasecmp;
 	}
-	if (vp == NULL || cp == NULL) {
-		return(NULL);
-	}
 	if (!isvarchar(*cp)) {
+		error(_("Invalid variable."));
 		return(NULL);	/* variable name must start with a valid variable character */
 	}
 	cp1 = cp;
@@ -355,25 +361,18 @@ char	*cp;
 			break;
 		}
 		if (i >= MAX_VAR_LEN) {
+			error(_("Variable name too long."));
 			return(NULL);
 		}
 		buf[i++] = *cp1++;
 	}
 	buf[i] = '\0';
 	if ((*strcmpfunc)(buf, INFINITY_NAME) == 0) {
+		error(_("Invalid variable."));
 		return(NULL);
 	} else if ((*strcmpfunc)(buf, "sign") == 0) {
 		vtmp = SIGN;
 		cp += strlen(buf);
-	} else if ((*strcmpfunc)(buf, "integer") == 0) {
-		vtmp = V_INTEGER;
-		cp += strlen(buf);
-	} else if ((*strcmpfunc)(buf, "temp") == 0) {
-		vtmp = V_TEMP;
-		cp += strlen(buf);
-	} else if ((*strcmpfunc)(buf, "answer") == 0) {
-		vtmp = V_ANSWER;
-		cp += strlen(buf);
 	} else {
 		if (strncasecmp(cp, "i#", 2) == 0) {
 			*vp = IMAGINARY;
@@ -392,6 +391,7 @@ char	*cp;
 				break;
 			}
 			if (i >= MAX_VAR_LEN) {
+				error(_("Variable name too long."));
 				return(NULL);
 			}
 			buf[i++] = *cp++;
@@ -415,7 +415,7 @@ char	*cp;
 			len = strlen(buf) + 1;
 			var_names[i] = (char *) malloc(len);
 			if (var_names[i] == NULL) {
-				error(_("Out of memory!  (can't malloc())."));
+				error(_("Out of memory (can't malloc(3) variable name)."));
 				return(NULL);
 			}
 			blt(var_names[i], buf, len);
@@ -426,6 +426,7 @@ char	*cp;
 	if (isdigit(*cp)) {
 		j = strtol(cp, &cp, 10);
 		if (j < 0 || j > MAX_SUBSCRIPT) {
+			error(_("Maximum subscript exceeded in special variable name."));
 			return(NULL);
 		}
 		if (vtmp == SIGN) {
@@ -460,8 +461,9 @@ char	*cp;	/* input string */
 		cp1--;
 		if (*cp1 == '\n' || *cp1 == '\r' || isspace(*cp1)) {
 			*cp1 = '\0';
-		} else
+		} else {
 			break;
+		}
 	}
 /* set point_flag to false if non-printable characters encountered */
 	for (cp1 = cp; *cp1; cp1++) {
