@@ -57,7 +57,21 @@ long		v;		/* variable to factor */
 {
 	if (*np == 0)
 		return;
-	simp_ssub(equation, np, v, 0.0, v == 0L, true, 2);
+	simp_ssub(equation, np, v, 0.0, v == 0, true, 2);
+}
+
+/*
+ * Convert expression with any algebraic fractions into a single fraction.
+ */
+frac_side(equation, np)
+token_type	*equation;	/* pointer to the beginning of equation side */
+int		*np;		/* pointer to length of equation side */
+{
+	if (*np == 0)
+		return;
+	do {
+		simpb_side(equation, np, false, 2);
+	} while (super_factor(equation, np, 2));
 }
 
 /*
@@ -68,14 +82,16 @@ calc_simp(equation, np)
 token_type	*equation;
 int		*np;
 {
-	in_calc_cmd = true;
+	approximate_roots = true;
 	subst_constants(equation, np);
 	simp_side(equation, np);
 	ufactor(equation, np);
 	factorv(equation, np, IMAGINARY);
+	ufactor(equation, np);
+	factorv(equation, np, IMAGINARY);
 	simp_side(equation, np);
 	uf_simp(equation, np);
-	in_calc_cmd = false;
+	approximate_roots = false;
 }
 
 /*
@@ -133,7 +149,7 @@ token_type	*equation;	/* pointer to the beginning of equation side to simplify *
 int		*np;		/* pointer to length of the equation side */
 int		quick_flag;	/* "simplify quick" option */
 {
-	int		flag, poly_flag = false;
+	int		flag, poly_flag = true;
 	jmp_buf		save_save;
 
 	if (*np <= 1)	/* no need to simplify a constant or a variable */
@@ -141,13 +157,7 @@ int		quick_flag;	/* "simplify quick" option */
 	partial_flag = 2;
 	simp_loop(equation, np);
 	poly_factor(equation, np);
-#if	false
-	do {
-		simp_ssub(equation, np, 0L, 0.0, false, true, 1);
-	} while (uf_power(equation, np));
-#else
 	simpb_side(equation, np, true, 1);
-#endif
 	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
 	rationalize(equation, np);
 	unsimp_power(equation, np);
@@ -185,7 +195,7 @@ int		quick_flag;	/* "simplify quick" option */
 		do {
 			simp_ssub(equation, np, 0L, 1.0, false, true, 1);
 		} while (uf_power(equation, np));
-	} while (super_factor(equation, np, 1));
+	} while (super_factor(equation, np, 2));
 	if (quick_flag) {
 do_quick:
 		uf_tsimp(equation, np);
@@ -887,7 +897,7 @@ int		iflag;		/* produce imaginary numbers flag */
 }
 
 /*
- * Do the arithmetic.
+ * Do the floating point arithmetic for Mathomatic.
  *
  * Return true if successful.
  */
@@ -899,8 +909,7 @@ int	op2;	/* operator 2 */
 double	k2;	/* operand 2 */
 {
 	int	op1;
-	double	d;
-	double	d1, d2;
+	double	d, d1, d2;
 
 	domain_check = false;
 	errno = 0;
@@ -946,13 +955,11 @@ double	k2;	/* operand 2 */
 				*k1p = *k1p / k2;
 			}
 		}
-		d1 = fabs(*k1p) * small_epsilon;
-		d = my_round(*k1p);
-		if (d != 0.0 && fabs(d - *k1p) < d1) {
-			*k1p = d;
-		}
 		break;
 	case MODULUS:
+		if (k2 == 0) {
+			debug_string(0, _("Warning: modulo 0 encountered, might be considered undefined."));
+		}
 #if	false	/* If fmod() always works.  It doesn't. */
 		*k1p = fmod(*k1p, k2);
 #else
@@ -976,7 +983,7 @@ double	k2;	/* operand 2 */
 			d = 1.0;	/* some people don't know 0^0 */
 		} else {
 			d = pow(*k1p, k2);
-			if (preserve_roots && !in_calc_cmd) {
+			if (preserve_roots && !approximate_roots) {
 				if (isfinite(k2) && fmod(k2, 1.0) != 0.0 && f_to_fraction(*k1p, &d1, &d2)) {
 					if (!f_to_fraction(d, &d1, &d2)) {
 						domain_check = false;
@@ -991,8 +998,8 @@ double	k2;	/* operand 2 */
 		break;
 	case FACTORIAL:
 #if	true	/* set this to false if lgamma() doesn't exist */
-		d = exp(lgamma(*k1p + 1.0));
-		if (errno) {	/* simply don't evaluate if error */
+		d = exp(lgamma(*k1p + 1.0)) * signgam;
+		if (errno) {	/* don't evaluate if overflow */
 			return false;
 		}
 #else
@@ -1050,7 +1057,7 @@ beginning:
 			if (calc((loc1 <= old_loc) ? &zero : &equation[loc1-1].token.operatr, &d, op, d2)) {
 				if (op == POWER && !domain_check) {
 					if (!f_to_fraction(d2, &numerator, &denominator)) {	/* if irrational power */
-						if (!iflag || (preserve_roots && !in_calc_cmd))
+						if (!iflag || (preserve_roots && !approximate_roots))
 							return modified;
 						cv.re = d;
 						cv.im = 0.0;
@@ -2002,9 +2009,8 @@ int		*np, loc, level;
 		}
 		for (k = j + 2; k < i;) {
 			if (equation[k].level == level && equation[k].token.operatr == TIMES) {
-				for (n = k + 2; n < i; n += 2)
-					if (equation[n].level <= level)
-						break;
+				for (n = k + 2; n < i && equation[n].level > level; n += 2)
+					;
 				n -= k;
 				blt(scratch, &equation[k], n * sizeof(token_type));
 				blt(&equation[j+n], &equation[j], (k - j) * sizeof(token_type));
@@ -2020,9 +2026,8 @@ int		*np, loc, level;
 	}
 	return modified;
 corrupt:
-	error(_("Expression is corrupt!"));
+	error(_("Expression is corrupt!  Please file a bug report."));
 	longjmp(jmp_save, 13);
-
 }
 
 /*
