@@ -693,10 +693,11 @@ int		*np;
 					equation[i+3].token.constant -= 1.0;
 					blt(&equation[i+2], &equation[i], (*np - i) * sizeof(token_type));
 					*np += 2;
-					equation[i+1].level = level - 1;
-					equation[i+1].kind = CONSTANT;
-					equation[i+1].token.constant = equation[i+3].token.constant;
-					i += 2;
+					i++;
+					equation[i].level = level - 1;
+					equation[i].kind = CONSTANT;
+					equation[i].token.constant = equation[i+2].token.constant;
+					i++;
 				} else {
 					equation[i].token.operatr = TIMES;
 					equation[i+1].token.constant = 1.0 / equation[i+1].token.constant;
